#!/bin/bash
## This script MUST be run on the TARGET machine in order to provide relevant information.
## This script assumes to be run by the root user (with /sbin in the path).

set -e

# get the root of the git repository (requires git to be installed)
GIT_ROOT=`git rev-parse --show-toplevel`
cd $GIT_ROOT

# define useful variables
source generation/00env.sh

get_partition_path () {
	# Return the path of the desired partition.
	# Exit and display an error if it finds 0 or several results.
	#
	# This function should be called ith exactly one argument: the desired partition type.
	# Its behavior is undefined otherwise.

	desired_partitions=$(fdisk -l | grep "$1")

	desired_partition_count=$(echo -e "$desired_partitions" | wc | awk '{print $1}')
	if [ -z "$desired_partitions" ]; then
		echo -e "No partition of type $1 (shouldbe 1). Exiting."
		exit
	elif [ "$desired_partition_count" -ne 1 ]; then
		echo -e "$desired_partition_count partitions of type $1 (should be 1). Exiting."
		exit
	else
		echo -e "$desired_partitions" | awk '{print $1}'
	fi
}

get_partition_id () {
        desired_partition_path=$(get_partition_path "$1")
        echo -e "$(findmnt -fn -o UUID "$desired_partition_path")"
}

get_default_interface () {
	default_results=$(ip route | grep default | wc | awk '{print $1}')
	if [ "$default_results" -ne 1 ]; then
		echo -e "$default_results default result(s) (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep default | sed 's|.*dev \([^ ]*\).*|\1|g')"
	fi
}

get_default_router () {
	default_results=$(ip route | grep default | wc | awk '{print $1'})
	if [ "$default_results" -ne 1 ]; then
		echo -e "$defaults_results default result(s) (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep default | sed 's|.*via \([^ ]*\).*|\1|g')"
	fi
}

get_default_ip () {
	default_interface=$(get_default_interface)
	interface_results=$(ip route | grep -v default | grep "$default_interface" | wc | awk '{print $1}')
	if [ "$interface_results" -ne 1 ]; then
		echo -e "$interface_results results for \"${default_interface}\" (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep -v default | grep "$default_interface" | sed 's|.*src \([^ ]*\).*|\1|g')"
	fi
}

get_default_netmask () {
	default_interface=$(get_default_interface)
	interface_results=$(ip route | grep -v default | grep "$default_interface" | wc | awk '{print $1}')
	if [ "$interface_results" -ne 1 ]; then
		echo -e "$interface_results results for \"${default_interface}\" (should be 1). Exiting."
		exit
	else
		echo -e "$(ip route | grep -v default | grep "$default_interface" | sed 's|^\([^ ]*\).*|\1|g')"
	fi
}

int_to_generic_netmask () {
	# get the mask size: the number right after the "/" (ex: <ipv4>/24 -> 24)
	mask_length=$1
	mask=""
	for i in {0..3}; do
		if [ ${mask_length} -ge 8 ]; then
			mask=${mask}255
		elif [ ${mask_length} -eq 0 ]; then
			mask=${mask}0
		else
			mask=${mask}$((256-2**(8-${mask_length})))
		fi
		if [ $i -ne 3 ]; then
			mask=${mask}.
		fi
		mask_length=$((mask_length-8))
	done
	echo -e "${mask}"
}

# Get information about the network
DEFAULT_INTERFACE=$(get_default_interface)
DEFAULT_ROUTER=$(get_default_router)
DEFAULT_IP=$(get_default_ip)
DEFAULT_NETMASK=$(get_default_netmask)
DEFAULT_NETMASK_NUMBER=$(echo -e "${DEFAULT_NETMASK}" | awk -F / '{print $2}')
FORMATTED_NETMASK=$(int_to_generic_netmask "${DEFAULT_NETMASK_NUMBER}")
TARGET_HOSTNAME=$(hostname)

# Get information about the partition layout
root_partition_type="Linux filesystem"
efi_partition_type="EFI System"
EFI_PARTITION_MOUNT_POINT=$(get_partition_path "$efi_partition_type")
ROOT_PARTITION_ID=$(get_partition_id "$root_partition_type")

echo "export DEFAULT_NETMASK=\"$DEFAULT_NETMASK\""
echo "export DEFAULT_NETMASK_NUMBER=\"$DEFAULT_NETMASK_NUMBER\""
echo "export DEFAULT_INTERFACE=\"$DEFAULT_INTERFACE\""
echo "export DEFAULT_ROUTER=\"$DEFAULT_ROUTER\""
echo "export DEFAULT_IP=\"$DEFAULT_IP\""
echo "export FORMATTED_NETMASK=\"$FORMATTED_NETMASK\""
echo "export TARGET_HOSTNAME=\"$TARGET_HOSTNAME\""
echo "export EFI_PARTITION_MOUNT_POINT=\"$EFI_PARTITION_MOUNT_POINT\""
echo "export ROOT_PARTITION_ID=\"$ROOT_PARTITION_ID\""
